#include "assets.h"
#include "camera.h"
#include "debug.h"
#include "input.h"
#include "physics_world.h"
#include "program.h"
#include "rigidbody.h"
#include "spritesheet.h"
#include "sprite.h"
#include "tilemap.h"
#include "player.h"
#include <assert.h>

static Spritesheet* spr_player_standing = NULL;
static Player* player = NULL;

static Level* level = NULL;

static Vector cam_speed = ZeroVector;

static
void cam_move_h(int val) {
    cam_speed.x = val * 0.1f;
}
static
void cam_move_v(int val) {
    cam_speed.y = -val * 0.1f;
}

static
void play() {
    spr_player_standing = spritesheet_load("assets/sprites/player.png", (IVector){128, 128});

    g_camera.fov = 40;

    player = malloc(sizeof(Player));
    player->transform.scale = (Vector){4.f, 4.f};
    store_asset(player, free);
    player->sprite = sprite_from_spritesheet(spr_player_standing, 0);
    player->rigidbody = rigidbody_make(Player_as_Transformable(player));
    sprite_set_origin(player->sprite, (Vector){0.25f, 1.f});
    player->shape = shape_new((Vector[]){
        {-.1f, -.75f},
        {0.1f, -.75f},
        {0.1f, 0.00f},
        {-.1f, 0.00f}
    }, 4);

    level = level_load("level_0");

    input_add_axis_action(SDL_SCANCODE_A, SDL_SCANCODE_D, &cam_move_h);
    input_add_axis_action(SDL_SCANCODE_S, SDL_SCANCODE_W, &cam_move_v);

    physics_world_add_entity(Player_as_PhysicsEntity(player));
}

static
void tick() {
    g_camera.transform.position = vaddf(g_camera.transform.position, cam_speed);
    player->transform.position = g_camera.transform.position;
}

static
void draw() {
    level_draw(level);
    sprite_entity_draw(Player_as_SpriteEntity(player));
}

int main(int argc, char* argv[]) {
    struct ProgramSettings config = {
        .target_fps = 120,
        .title = "fencer",
        .view_resolution = {1920, 1080},
        .on_play = &play,
        .on_tick = &tick,
        .on_draw = &draw
    };
    return program_run(&config);
}
